@@ -21,10 +21,15 @@
  * Authors: Glynn Foster <glynn.foster@sun.com>
  */
 
-#include "config.h"
+#include <config.h>
+
 #include "zenity.h"
+#include <stdlib.h>
 #include <popt.h>
 #include <langinfo.h>
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
 
 typedef enum {
   MODE_CALENDAR,
@@ -88,7 +93,9 @@ enum {
   OPTION_ERRORTEXT,
   OPTION_INFOTEXT,
   OPTION_FILENAME,
+  OPTION_MULTIFILE,
   OPTION_TEXTFILENAME,
+  OPTION_LISTTEXT,
   OPTION_COLUMN,
   OPTION_SEPERATOR,
   OPTION_LISTEDIT,
@@ -97,6 +104,8 @@ enum {
   OPTION_PROGRESSTEXT,
   OPTION_PERCENTAGE,
   OPTION_PULSATE,
+  OPTION_AUTOCLOSE,
+  OPTION_PRINTCOLUMN,
   OPTION_QUESTIONTEXT,
   OPTION_WARNINGTEXT,
   OPTION_ABOUT,
@@ -105,10 +114,10 @@ enum {
 };
 
 static void zenity_parse_options_callback (poptContext              ctx,
-    enum poptCallbackReason  reason,
-    const struct poptOption *opt,
-    const char              *arg,
-    void                    *data);
+					   enum poptCallbackReason  reason,
+					   const struct poptOption *opt,
+					   const char              *arg,
+					   void                    *data);
 
 struct poptOption options[] = {
   {
@@ -422,6 +431,24 @@ struct poptOption file_selection_options[] = {
     N_("Set the filename"),
     N_("FILENAME")
   },
+  {
+    "multiple",
+    '\0',
+    POPT_ARG_NONE,
+    NULL,
+    OPTION_MULTIFILE,
+    N_("Allow multiple files to be selected"),
+    NULL
+  },
+  {
+    "separator",
+    '\0',
+    POPT_ARG_STRING,
+    NULL,
+    OPTION_SEPERATOR,
+    N_("Set output separator character."),
+    N_("SEPARATOR")
+  },
   POPT_TABLEEND
 };
 
@@ -435,6 +462,15 @@ struct poptOption list_options[] = {
     NULL,
     NULL
   },
+  {
+    "text",
+    '\0',
+    POPT_ARG_STRING,
+    NULL,
+    OPTION_LISTTEXT,
+    N_("Set the dialog text"),
+    NULL
+  },
   {
     "column",
     '\0',
@@ -469,7 +505,7 @@ struct poptOption list_options[] = {
     NULL,
     OPTION_SEPERATOR,
     N_("Set output separator character"),
-    NULL
+    N_("SEPARATOR")
   },
   {
     "editable",
@@ -480,6 +516,15 @@ struct poptOption list_options[] = {
     N_("Allow changes to text"),
     NULL
   },
+  {
+    "print-column",
+    '\0',
+    POPT_ARG_STRING,
+    NULL,
+    OPTION_PRINTCOLUMN,
+    N_("Print a specific column (Default is 1. 'ALL' can be used to print all columns)"),
+    NULL
+  },
   POPT_TABLEEND
 };
 
@@ -520,6 +565,16 @@ struct poptOption progress_options[] = {
     N_("Pulsate progress bar"),
     NULL
   },
+  {
+    "auto-close",
+    '\0',
+    POPT_ARG_NONE,
+    NULL,
+    OPTION_AUTOCLOSE,
+    /* xgettext: no-c-format */
+    N_("Dismiss the dialog when 100% has been reached"),
+    NULL
+  },
   POPT_TABLEEND
 };
 
@@ -908,19 +963,26 @@ zenity_init_parsing_options (void) {
   /* Give some sensible defaults */
   results->data->width = -1;
   results->data->height = -1;
-  results->calendar_data->date_format = g_strdup (nl_langinfo (D_FMT));
+  results->calendar_data->date_format = g_locale_to_utf8 (nl_langinfo (D_FMT),
+							  -1,
+							  NULL, NULL, NULL);
   results->calendar_data->day = 0;
   results->calendar_data->month = 0;
   results->calendar_data->year = 0;
   results->calendar_data->dialog_text = NULL;
+  results->file_data->multi = FALSE;
+  results->file_data->separator = g_strdup ("|");
   results->text_data->editable = FALSE;
-  results->tree_data->separator = g_strdup ("/");
+  results->tree_data->separator = g_strdup ("|");
   results->progress_data->percentage = -1;
   results->progress_data->pulsate = FALSE;
+  results->progress_data->autoclose = FALSE;
   results->entry_data->visible = TRUE;
+  results->tree_data->dialog_text = NULL;
   results->tree_data->checkbox = FALSE;
   results->tree_data->radiobox = FALSE;
   results->tree_data->editable = FALSE;
+  results->tree_data->print_column = NULL;
 }
 
 static void
@@ -956,16 +1018,21 @@ zenity_free_parsing_options (void) {
     case MODE_FILE:
       if (results->file_data->uri)
         g_free (results->file_data->uri);
+	g_free (results->file_data->separator);
       break;
     case MODE_TEXTINFO:
       if (results->text_data->uri)
         g_free (results->text_data->uri);
       break;
     case MODE_LIST:
+      if (results->tree_data->dialog_text)
+        g_free (results->tree_data->dialog_text);
       if (results->tree_data->columns)
         g_slist_foreach (results->tree_data->columns, (GFunc) g_free, NULL);
       if (results->tree_data->separator)
         g_free (results->tree_data->separator);
+      if (results->tree_data->print_column)
+        g_free (results->tree_data->print_column);
       break;
     default:
       break;
@@ -974,12 +1041,13 @@ zenity_free_parsing_options (void) {
 
 gint 
 main (gint argc, gchar **argv) {
-  ZenityData *general;
-  ZenityCalendarData *cal_data;
   poptContext ctx;
-  gchar **args;
   gint nextopt, retval;
 
+#ifdef HAVE_LOCALE_H
+  setlocale(LC_ALL,"");
+#endif
+
   bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
   bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
   textdomain(GETTEXT_PACKAGE);
@@ -1000,7 +1068,7 @@ main (gint argc, gchar **argv) {
     /*nothing*/;
 
   if (nextopt != -1) {
-    g_printerr (_("%s in an invalid option. See zenity --help for more details\n"),
+    g_printerr (_("%s is an invalid option. See 'zenity --help' for more details\n"),
                   poptBadOption (ctx, 0));
     zenity_free_parsing_options (); 
     exit (-1); 
@@ -1008,11 +1076,12 @@ main (gint argc, gchar **argv) {
   gtk_init (&argc, &argv);
 
   /* 
-  if (argc < 2) {
-    g_printerr (_("You must specify more arguments. See zenity --help for more details\n"));
-    zenity_free_parsing_options ();
-    exit (-1);
-  } */
+   * if (argc < 2) {
+   *   g_printerr (_("You must specify more arguments. See zenity --help for more details\n"));
+   *   zenity_free_parsing_options (); 
+   *   exit (-1); 
+   * }
+   */
 
   switch (results->mode) {
     case MODE_CALENDAR:
@@ -1092,7 +1161,7 @@ zenity_parse_options_callback (poptContext              ctx,
   static gint parse_option_text = 0;
   static gint parse_option_file = 0;
   static gint parse_option_editable = 0;
-
+  
   if (reason == POPT_CALLBACK_REASON_POST)
     return;
   else if (reason != POPT_CALLBACK_REASON_OPTION)
@@ -1193,6 +1262,7 @@ zenity_parse_options_callback (poptContext              ctx,
     case OPTION_ERRORTEXT: 
     case OPTION_QUESTIONTEXT: 
     case OPTION_PROGRESSTEXT: 
+    case OPTION_LISTTEXT: 
     case OPTION_WARNINGTEXT: 
       
       /* FIXME: This is an ugly hack because of the way the poptOptions are 
@@ -1200,24 +1270,32 @@ zenity_parse_options_callback (poptContext              ctx,
        * parse_options_callback gets called for each option. Suckage 
        */
 
-      if (parse_option_text > 6) 
+      if (parse_option_text > 7) 
         zenity_error ("--text", ERROR_DUPLICATE); 
-      
+     
       switch (results->mode) { 
         case MODE_CALENDAR: 
-          results->calendar_data->dialog_text = g_strdup (arg); 
+          results->calendar_data->dialog_text = g_locale_to_utf8 (g_strcompress (arg),
+								  -1, NULL, NULL, NULL);
           break; 
         case MODE_ENTRY: 
-          results->entry_data->dialog_text = g_strdup (arg); 
+          results->entry_data->dialog_text = g_locale_to_utf8 (g_strcompress (arg),
+							       -1, NULL, NULL, NULL);
           break; 
         case MODE_ERROR: 
         case MODE_QUESTION: 
         case MODE_WARNING: 
         case MODE_INFO: 
-          results->msg_data->dialog_text = g_strdup (arg); 
+          results->msg_data->dialog_text = g_locale_to_utf8 (g_strcompress (arg),
+							     -1, NULL, NULL, NULL);
           break; 
         case MODE_PROGRESS: 
-          results->progress_data->dialog_text = g_strdup (arg); 
+          results->progress_data->dialog_text = g_locale_to_utf8 (g_strcompress (arg),
+								  -1, NULL, NULL, NULL);
+          break; 
+        case MODE_LIST: 
+          results->tree_data->dialog_text = g_locale_to_utf8 (g_strcompress (arg),
+						              -1, NULL, NULL, NULL);
           break; 
         default: 
           zenity_error ("--text", ERROR_SUPPORT); 
@@ -1287,7 +1365,7 @@ zenity_parse_options_callback (poptContext              ctx,
        * parse_options_callback gets called for each option. Suckage 
        */ 
       
-      if (parse_option_file > 2) 
+      if (parse_option_editable > 2) 
         zenity_error ("--editable", ERROR_DUPLICATE); 
       
       switch (results->mode) { 
@@ -1325,6 +1403,12 @@ zenity_parse_options_callback (poptContext              ctx,
       } 
       parse_option_file++; 
       break; 
+    case OPTION_MULTIFILE:
+      if (results->mode != MODE_FILE)
+        zenity_error ("--multiple", ERROR_SUPPORT);
+
+      results->file_data->multi = TRUE;
+      break;
     case OPTION_COLUMN: 
       if (results->mode != MODE_LIST) 
         zenity_error ("--column", ERROR_SUPPORT); 
@@ -1333,31 +1417,37 @@ zenity_parse_options_callback (poptContext              ctx,
       break; 
     case OPTION_CHECKLIST: 
       if (results->mode != MODE_LIST) 
-        zenity_error ("--checkbox", ERROR_SUPPORT); 
+        zenity_error ("--checklist", ERROR_SUPPORT); 
       
       if (results->tree_data->checkbox) 
-        zenity_error ("--checkbox", ERROR_DUPLICATE); 
+        zenity_error ("--checklist", ERROR_DUPLICATE); 
       
       results->tree_data->checkbox = TRUE; 
       break; 
     case OPTION_RADIOLIST: 
       if (results->mode != MODE_LIST) 
-        zenity_error ("--radiobox", ERROR_SUPPORT); 
+        zenity_error ("--radiolist", ERROR_SUPPORT); 
       if (results->tree_data->radiobox) 
-        zenity_error ("--radiobox", ERROR_DUPLICATE); 
+        zenity_error ("--radiolist", ERROR_DUPLICATE); 
       
       results->tree_data->radiobox = TRUE; 
       break; 
     case OPTION_SEPERATOR: 
-      if (results->mode != MODE_LIST) 
-        zenity_error ("--separator", ERROR_SUPPORT); 
-      
-      if (parse_option_separator) 
-        zenity_error ("--separator", ERROR_DUPLICATE); 
-      
-      results->tree_data->separator = g_strdup (arg); 
-      parse_option_separator = TRUE; 
-      break; 
+      if (parse_option_separator > 2)
+	zenity_error ("--separator", ERROR_DUPLICATE);
+      switch (results->mode) {
+	case MODE_LIST: 
+      	  results->tree_data->separator = g_strdup (g_strcompress (arg)); 
+      	  parse_option_separator++; 
+	  break;
+	case MODE_FILE:
+	  results->file_data->separator = g_strdup (g_strcompress (arg));
+	  parse_option_separator++;
+	  break;
+	default:
+	  zenity_error ("--separator", ERROR_SUPPORT);
+	}
+      break;
     case OPTION_PERCENTAGE: 
       if (results->mode != MODE_PROGRESS) 
         zenity_error ("--percentage", ERROR_SUPPORT); 
@@ -1373,6 +1463,18 @@ zenity_parse_options_callback (poptContext              ctx,
       
       results->progress_data->pulsate = TRUE; 
       break; 
+    case OPTION_AUTOCLOSE:
+      if (results->mode != MODE_PROGRESS)
+        zenity_error ("--auto-close", ERROR_SUPPORT);
+
+      results->progress_data->autoclose = TRUE;
+      break;
+    case OPTION_PRINTCOLUMN:
+      if (results->mode != MODE_LIST)
+        zenity_error ("--print-column", ERROR_SUPPORT);
+
+      results->tree_data->print_column = g_strdup (arg);
+      break;
     case OPTION_ABOUT: 
       if (results->mode != MODE_LAST) 
         zenity_error (NULL, ERROR_DIALOG); 
